from qt import *
from qtcanvas import *
from math import *

yu, yh = 28, 30
sze = yh*6+20
xo, y2 = yu * sqrt(3)/2, yu/2
colors = {"W": Qt.blue, "R": Qt.red, "G": Qt.lightGray, "0": QColor(255, 255, 224)}
holes = "D3", "C4", "E5"

class Hexxagon(QDialog):
    _public_methods_ = ["Move", "Undo", "Reset", "White", "Red", "Empty", "State"]
    _reg_clsid_ = "{48CD4D6B-F85F-4F19-9284-3E89ABEF4DC2}"
    _reg_progid_ = "Python.Hexxagon"
    _reg_desc_ = "Python Hexxagon Drawer"

    def __init__(self):
        self.app = QApplication([])
        QDialog.__init__(self, None, "Hexxagon", False, Qt.WStyle_Customize + Qt.WStyle_NormalBorder + Qt.WStyle_Title + Qt.WStyle_SysMenu + Qt.WStyle_Minimize)
        self.setCaption("Qt Hexxagon")
        layout = QVBoxLayout(self)
        self.cnHexx = QCanvas(self)
        self.cnHexx.setBackgroundColor(colors["0"])
        self.cnHexx.resize(2*sze, 2*sze)
        cvHexx = QCanvasView(self.cnHexx, self)
        cvHexx.resize(2*sze+10, 2*sze+10)
        self.resize(2*sze+10, 2*sze+10)

        self.cells = {}
        for y,r in zip(range(-3, 4), "ABCDEFG"):
            oy = sze + y*yh*3/2.
            lx = -3+abs(y)/2
            self.putt(r, sze + yh*sqrt(3)*(lx + (y % 2) / 2.) - yh - 10, oy - 6)
            for x,c in zip(range(lx, 4-(abs(y)+1)/2), "12345678"[max(0, y):]):
                cell = QCanvasPolygon(self.cnHexx)
                ox = sze + yh*sqrt(3)*(x + (y % 2) / 2.)
                cell.setPoints(QPointArray([ox+xo, oy+y2, ox, oy+yu, ox-xo, oy+y2, ox-xo, oy-y2, ox, oy-yu, ox+xo, oy-y2]))
                cell.show()
                self.cells[r+c] = cell
                if y==-3:
                    self.putt(c, ox + yu/2, oy - yu*1.3)
            if -3 < y <= 0:
                self.putt(str(y+7), ox + yu/2, oy - yu*1.5)
        self.Reset()

        self.app.setMainWidget(self)
        self.show()

    def putt(self, s, x, y):
        t = QCanvasText(s, self.cnHexx)
        t.move(x, y)
        t.show()
        
    def recolor(self):
        for k, v in self.cells.items():
            v.setBrush(QBrush(colors[self.state.get(k, False)  or  k in holes and "0"  or "G"]))
        self.cnHexx.update()
        self.app.processEvents()
            
    def Reset(self):
        self.state = {"A1": "W", "D7": "W", "G4": "W", "A4": "R", "G7": "R", "D1": "R"}
        self.history = [self.state]
        self.recolor()

    def Move(self, move, draw=False):
        src, dst = move[:2], move[2:]
        src, dst = src.upper(), dst.upper()
        src_y, src_x = ord(src[0]), ord(src[1])
        dst_y, dst_x = ord(dst[0]), ord(dst[1])
        neighs = (dst_y-1, dst_x-1), (dst_y-1, dst_x), (dst_y, dst_x-1), (dst_y, dst_x+1), (dst_y+1, dst_x), (dst_y+1, dst_x+1)

        state = self.state = dict(self.history[-1]) # make a copy!
        me = state.get(src, False)
        if not me or state.get(dst, False) or not dst in self.cells:
            return
        him = "R" if me == "W" else "W"

        state[dst] = me
        if not (src_y, src_x) in neighs:
            del state[src]
        for n in neighs:
            n = chr(n[0])+chr(n[1])
            if state.get(n, None) == him:
                state[n] = me;
        self.history.append(state)
        if draw: self.recolor()

    def Undo(self, draw=False):
        if len(self.history) > 1:
            del self.history[-1]
            self.state = self.history[-1]
            if draw: self.recolor()

    def White(self):
        return [k for k, v in self.state.items() if v == "W"]

    def Red(self):
        return [k for k, v in self.state.items() if v == "R"]

    def Empty(self):
        return [k for k in self.cells if k not in self.state and k not in holes]

    def State(self):
        return self.state.items()
       
        
if __name__=="__main__":
    import win32com.server, win32com.server.register
    win32com.server.register.UseCommandLine(Hexxagon)

#a = Hexxagon()
#a.Move("A1", "A2")
#a.Move("D1", "B1")
#a.app.exec_loop()

# import win32com.client; a = win32com.client.Dispatch("Python.Hexxagon")